package wifi;

import java.net.NetworkInterface;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
//import android.content.IntentFilter;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.ScanResult;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.BA.Permissions;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;

/*----- Class MLwifi ------------------------------------------------------------*/

@ShortName("MLwifi")
@Version(2.18f)
@Author("Jem Miller - Missing Link Software")
@Permissions(values = {"android.permission.ACCESS_NETWORK_STATE","android.permission.ACCESS_WIFI_STATE","android.permission.CHANGE_WIFI_STATE","android.permission.ACCESS_FINE_LOCATION","android.permission.INTERNET"})

public class MLwifi {
	
/**
 * Turns WIFI On or Off. Pass True to Enable WIFI or False to Disable.
 */
public void EnableWifi(boolean Enabled) {
	Context context = BA.applicationContext;
	try {
		WifiManager wifi = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
		wifi.setWifiEnabled(Enabled);	
	}
	catch (Exception e) { }
}

/**
 * Returns whether WIFI is Enabled (true) or Disabled (false)
 */
public boolean isWifiEnabled() {
	Context context = BA.applicationContext;
	try {
		WifiManager wifi = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
		return wifi.isWifiEnabled();	
	}
	catch (Exception e) { return false; }
}

/**
 * Returns the current devices Android API (the version of Android running on the device.) as an integer such as 19, 21, 23, etc.
 */
public static int APIversion() {
	int currentapiVersion = android.os.Build.VERSION.SDK_INT;

	/*if (currentapiVersion >= android.os.Build.VERSION_CODES.LOLLIPOP) {
    	// Do something for Lollipop and above versions
	} else {
    	// do something for phones running an SDK before lollipop
	}*/
	return currentapiVersion;
}

/**
 * Returns MAC Address of the current interface or empty string
 */
public static String MACAddress() {
	if (APIversion() >= android.os.Build.VERSION_CODES.M) {
		try {
			String interfaceName = "wlan0";
			List<NetworkInterface> interfaces = Collections.list(NetworkInterface.getNetworkInterfaces());
			for (NetworkInterface intf : interfaces) {
				if (!intf.getName().equalsIgnoreCase(interfaceName)) { continue; }

				byte[] mac = intf.getHardwareAddress();
				if (mac==null) { return ""; }

				StringBuilder buf = new StringBuilder();
				for (byte aMac : mac) { buf.append(String.format("%02X:", aMac)); }
				if (buf.length()>0) { buf.deleteCharAt(buf.length() - 1); }
				return buf.toString();
			}
		} catch (Exception ex) { return ""; } 
		return "";
	}
	else {
		Context context = BA.applicationContext;
		try {
			WifiManager manager = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
			WifiInfo info = manager.getConnectionInfo();
			return info.getMacAddress();
		} catch (Exception ex) { return ""; } 
	}
}

/**
 * Get IP Address from the WIFI interface or empty string
 */
public static String IpAddress() {
    Context context = BA.applicationContext;
    try {
    	WifiManager wifiManager = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
    	WifiInfo wifiInfo = wifiManager.getConnectionInfo();
    	int ipAddress = wifiInfo.getIpAddress();
    	String ip = null;
    	ip = String.format("%d.%d.%d.%d",
			(ipAddress & 0xff), (ipAddress >> 8 & 0xff), (ipAddress >> 16 & 0xff), (ipAddress >> 24 & 0xff));
    	return ip;
	} catch (Exception ex) { return ""; } 
}

/**
 * Returns the current SSID or empty string
 */
public static String SSID() {
	Context context = BA.applicationContext;
	try {
		String st = "";
		WifiManager wifiManager = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
		WifiInfo wifiInfo = wifiManager.getConnectionInfo();
		st = wifiInfo.getSSID().replace('"', ' ');
		return st.trim();
	} catch (Exception ex) { return ""; } 
}

/**
 * Returns the WIFI signal strength in dBm as an integer
 * Returns 0 on error
 */
public int WifiSignal() {
	Context context = BA.applicationContext;	
	try {
		WifiManager wifiManager = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);
		int linkSpeed = wifiManager.getConnectionInfo().getRssi();
		return linkSpeed;
	}
	catch (Exception e) { return 0; }
}

/**
 * Returns the WIFI signal strength in percentage as an integer (0 to 100) 
 * Returns 0 on error.
 */
public int WifiStrength() {
	Context context = BA.applicationContext;
    try {
    	WifiManager wifiManager = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);
        int rssi = wifiManager.getConnectionInfo().getRssi();
        int level = WifiManager.calculateSignalLevel(rssi, 101);  //0..100
        //int percentage = (int) ((level/10.0)*100);
        return level;  //percentage;
    }
    catch (Exception e) { return 0; }
}

/**
 * Calculates WIFI signal percentage from WIFI signal level as an integer (0 to 100)
 * Signal Range: -100db -> 0%, -50db -> 100%
 */
public int CalcWifiPct(int WifiSignal) {
	int MIN_RSSI = -100, MAX_RSSI = -50;
	if (WifiSignal <= MIN_RSSI) return 0;
	else if (WifiSignal >= MAX_RSSI) return 100;
	else return 100*((WifiSignal - MIN_RSSI)/(MAX_RSSI - MIN_RSSI));
}

/**
 * Returns the WIFI signal strength in percentage as an integer (0 to 100) 
 * Returns 0 on error.
 */
public int WifiSignalPct() {
	return CalcWifiPct( WifiSignal() );
}

/**
 * Returns the LinkSpeed of the connected WIFI access point
 * Returns 0 on error.
 */
public int WifiLinkSpeed() {
	Context context = BA.applicationContext;   
	try {
		WifiManager wifiManager = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);
		WifiInfo wi = wifiManager.getConnectionInfo();   
		return wi.getLinkSpeed();
    }
    catch (Exception e) { return 0; }
}
   
/**
 * Checks if WIFI is connected. This only checks for WIFI connection to a router.
 * It does NOT test for Internet availability. Use isOnLine() for that.
 * 
 * Returns True if WIFI is connected or False if 2G, 3G, 4G, WIMAX or no connection.
 * 
 * This method uses "getNetworkInfo()" that was deprecated in API level 23.
 */
public boolean isWifiConnectedOld() {
	Context context = BA.applicationContext;
	try {
		ConnectivityManager connManager = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
		@SuppressWarnings("deprecation")
		NetworkInfo mWifi = connManager.getNetworkInfo(ConnectivityManager.TYPE_WIFI);
		return mWifi.isConnected();
    }
    catch (Exception e) { return false; }
}

/**
 * Checks if WIFI is connected. This only checks for WIFI connection to a router.
 * It does NOT test for Internet availability.
 * 
 * Returns True if WIFI is connected or False if 2G, 3G, 4G, WIMAX or no connection.
 */
public boolean isWifiConnected() {
	Context context = BA.applicationContext;
	try {
		ConnectivityManager connectivityManager 
			= (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
		NetworkInfo activeNetworkInfo = connectivityManager.getActiveNetworkInfo();	
		return (activeNetworkInfo != null  && activeNetworkInfo.getType() == ConnectivityManager.TYPE_WIFI && 
				activeNetworkInfo.isConnected() && activeNetworkInfo.isAvailable());
    }
    catch (Exception e) { return false; }
}

/**
 * Checks if Phone Data is connected. This only checks for connection to a phone network.
 * It does NOT test for Internet availability.
 * 
 * Returns True if Phone Data is connected or False if WIFI, WIMAX or no connection.
 */
public boolean isDataConnected() {
	Context context = BA.applicationContext;
	try {
		ConnectivityManager connectivityManager 
			= (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
		NetworkInfo activeNetworkInfo = connectivityManager.getActiveNetworkInfo();	
		return (activeNetworkInfo != null  && activeNetworkInfo.getType() == ConnectivityManager.TYPE_MOBILE && 
				activeNetworkInfo.isConnected() && activeNetworkInfo.isAvailable());
    }
    catch (Exception e) { return false; }
}

/**
 * Returns the Type (0:MOBILE, 1:WIFI, 6:WIMAX, 7:BLUETOOTH, 9:ETHERNET, etc) of the current Active Network or -1.
 */
public int ActiveNetworkType() {
	Context context = BA.applicationContext;
	try {
		ConnectivityManager connectivityManager 
			= (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
		NetworkInfo activeNetworkInfo = connectivityManager.getActiveNetworkInfo();
		if (activeNetworkInfo != null  && activeNetworkInfo.isConnected() && activeNetworkInfo.isAvailable())
			return activeNetworkInfo.getType();
		else return -1;
    }
    catch (Exception e) { return -1; }
}

/**
 * Returns the Type Name (WIFI, MOBILE, etc) of the current Active Network or empty string.
 */
public String ActiveNetworkTypeName() {
	Context context = BA.applicationContext;
	try {
		ConnectivityManager connectivityManager 
			= (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
		NetworkInfo activeNetworkInfo = connectivityManager.getActiveNetworkInfo();
		if (activeNetworkInfo != null  && activeNetworkInfo.isConnected() && activeNetworkInfo.isAvailable())
			return activeNetworkInfo.getTypeName();
		else return "";
    }
    catch (Exception e) { return ""; }
}

/**
 * Checks for Internet connection.
 * 
 * This method does not check or care what type of network is used.
 * It can be WIFI, 2G, 3G, 4g, WIMAX, etc. It just tests for Internet connectivity.
 * 
 * Returns True if Internet is available.
 * 
 * ERROR. This method only checks if the device is connected to any network (WIFI, Phone Data, etc),
 * but the network can be not connected to Internet (router ADSL not connected to phone line, etc).
 */
public boolean isOnline() {
	Context context = BA.applicationContext;
	try {
		ConnectivityManager connectivityManager 
			= (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
		NetworkInfo activeNetworkInfo = connectivityManager.getActiveNetworkInfo();
		return (activeNetworkInfo != null && activeNetworkInfo.isConnected() && activeNetworkInfo.isAvailable());
    }
    catch (Exception e) { return false; }
}

/**
 * Checks for Internet connection by sending a Ping to Google DNS server (Ip: 8.8.8.8).
 * 
 * This method does not check or care what type of network is used.
 * It can be WIFI, 2G, 3G, 4g, WIMAX, etc. It just tests for Internet connectivity.
 * 
 * Returns True if Internet is available.
 */
public boolean isOnlinePing() {
	return isOnlinePing2("8.8.8.8");
}
/**
 * Checks for Internet connection by sending a Ping to IpHost (Ip or Host name).
 * 
 * This method does not check or care what type of network is used.
 * It can be WIFI, 2G, 3G, 4g, WIMAX, etc. It just tests for Internet connectivity.
 * 
 * Returns True if Internet is available.
 */
public boolean isOnlinePing2(String IpHost) {
	if (!isOnline()) return false;
	
    Runtime runtime = Runtime.getRuntime();
    try {
    	Process ipProcess = runtime.exec("/system/bin/ping -c 1 " + IpHost.trim());
        int     exitValue = ipProcess.waitFor();
        return (exitValue == 0);
    } catch (Exception e) { return false; } 
}

/**
 * Adds an entry to the WIFI manager and saves the configuration of the new entry onto the device.
 *
 * ssid: Access Point SSID.
 * security: Access Point security type (0:NONE, 1:WEP, 2:WAP).
 * Password - Access Point password/phrase ("" for an open network).
 * 
 * Returns True if success
 */
public static boolean addWifiAP(String ssid, int security, String password) {
	Context context = BA.applicationContext;
   	try {
		WifiConfiguration wc = new WifiConfiguration();
		//if (wc == null)  return false; 

		BA.Log("ssid 1: " + ssid );
		if (ssid.startsWith("\"")) 			
			ssid = "\"".concat(ssid).concat("\"");
		wc.SSID = ssid;
		BA.Log("ssid 2: " + ssid );
		
		//wc.SSID = "\"".concat(ssid).concat("\"");
		wc.hiddenSSID = false;
		wc.priority=40;
		wc.status = WifiConfiguration.Status.DISABLED;    
			    
		wc.allowedProtocols.set(WifiConfiguration.Protocol.RSN);
		wc.allowedProtocols.set(WifiConfiguration.Protocol.WPA);
		wc.allowedPairwiseCiphers.set(WifiConfiguration.PairwiseCipher.CCMP);
		wc.allowedPairwiseCiphers.set(WifiConfiguration.PairwiseCipher.TKIP);
		wc.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.WEP40);
		wc.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.WEP104);
		wc.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.CCMP);
		wc.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.TKIP);

		if (security == 2) {  //WAP, WAP2
			wc.allowedKeyManagement.set(WifiConfiguration.KeyMgmt.WPA_PSK);
	        wc.preSharedKey = "\"".concat(password).concat("\"");	
		}
		else if (security == 1) {  //WEP
			wc.allowedKeyManagement.set(WifiConfiguration.KeyMgmt.NONE);
			wc.allowedAuthAlgorithms.set(WifiConfiguration.AuthAlgorithm.OPEN);
			wc.allowedAuthAlgorithms.set(WifiConfiguration.AuthAlgorithm.SHARED);
			if (isHex(password)) wc.wepKeys[0] = password;
			else wc.wepKeys[0] = "\"".concat(password).concat("\"");
			wc.wepTxKeyIndex = 0;
		}
		else {	//NONE
			wc.allowedKeyManagement.set(WifiConfiguration.KeyMgmt.NONE);
			wc.allowedAuthAlgorithms.clear();
		}

       	WifiManager wm = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);
		if (wm == null) return false;
			
		int NetId = wm.addNetwork(wc);
		//BA.Log("add Network returned: " + NetId );
		if (NetId <= 0) return false;

		boolean St = wm.enableNetwork(NetId, true);        
		//BA.Log("enableNetwork returned: " + St );
		wm.saveConfiguration();
       	return St;
   	} catch (Exception e) { return false; }
}
	   
private static boolean isHex (String in) {
    try {
    	// try to parse the string to an integer, using 16 as radix
        @SuppressWarnings("unused")
		int t = Integer.parseInt(in, 16);
        // parsing succeeded, string is valid hex number
        return true;
    } 	       
    catch (NumberFormatException e) {
    	// parsing failed, string is not a valid hex number
        return false;
    }    
}
	/*----- Class MLscan ------------------------------------------------------------*/

@Events(values = {"ScanDone(Results() As String, Count As Int)" })
@ShortName("MLScan") 

public static class MLWifiScanner {
	WifiManager WifiObj;
	WifiScanReceiver wifiReciever;
	private String eventname = "";
	protected List<ScanResult> Results = null;
	BA _ba = null;
	   
	/**
	 * A string array containing all scanned access points in the area of the device.
	 * These are comma separated entries consisting of the following:
	 * 
	 * SSID, Security type, signal strength, and "Saved" if this network was previously 
	 * saved for use on this device.
	 */
	public String wifis[];
	   
	/**
	 * Initializes all objects and scans for WIFI networks. Builds a string array 
	 * of results (wifis) that can be accessed directly or with a call to updateWifiList.
	 * THIS METHOD SHOULD ONLY BE CALLED ONCE! 
	 */
	public void startScan(BA ba, String EventName) {
		Context context = BA.applicationContext;
		   
		if (_ba == null) {
			_ba = ba;
		   
			WifiObj = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);
			eventname = EventName.toLowerCase();
	      
			wifiReciever = new WifiScanReceiver(this);
	      
			/*if (WifiObj.isWifiEnabled() && WifiObj != null) {
		      	context.registerReceiver(wifiReciever, new IntentFilter(WifiManager.SCAN_RESULTS_AVAILABLE_ACTION));
    	      	WifiObj.startScan();
	          }*/
		}
		   
	    if (!WifiObj.isWifiEnabled()) WifiObj.setWifiEnabled(true);
	      
	    updateWifiList(ba);
	}
	   	   
	/**
	 * Call only after startScan has already been called and a string array of Access Points h
	 * as been built. This method returns a string array containing the access points from the 
	 * most recent scan, including background scans done automatically by the Android WifiManager. 
	 * 
	 * Calls the ScanDone event when a new list is ready. 
	 * 
	 * Returns the string array of comma separated wifi access points by SSID. Each entry may
	 * contain the following:
	 * SSID, Security type, Signal level (in DB's), and the word "Saved" if the network was
	 * previously saved for use on this device.
	 */
	public void updateWifiList(BA ba) {
	    Context context = BA.applicationContext;
	    	 
		if (WifiObj != null && WifiObj.isWifiEnabled()) {
			if (this.wifiReciever == null) {
				this.wifiReciever = new WifiScanReceiver(this);
			}      
   		    context.registerReceiver(wifiReciever, new IntentFilter(WifiManager.SCAN_RESULTS_AVAILABLE_ACTION)); 
   		    
		    WifiObj.startScan();
		}		   
	}
	   
/*----- Class WifiScanReceiver ------------------------------------*/

	private class WifiScanReceiver extends BroadcastReceiver {
	MLWifiScanner tScan;
	      
	public WifiScanReceiver(MLWifiScanner tmpScan) {
		tScan = tmpScan;
	}
	      
	@Override
	public void onReceive(Context c, Intent intent) {
		String ssid = "", tmp = "";
	    tScan.Results = tScan.WifiObj.getScanResults();
	    tScan.wifis = new String[tScan.Results.size()];
	         
	    for (int i = 0; i < tScan.Results.size(); i++) {
	    	wifis[i] = tScan.Results.get(i).SSID;
	        ssid = tScan.wifis[i];
	        tmp = tScan.Results.get(i).capabilities;
	        int lev = tScan.Results.get(i).level;
	        	 
	        if (tmp.contains("WPA2")) 
	        	tScan.wifis[i] = tScan.wifis[i] + ",WPA2" + "," + String.valueOf(lev);
	        else
	        	if (tmp.contains("WPA"))
	        		tScan.wifis[i] = tScan.wifis[i] + ",WPA" + "," + String.valueOf(lev);
		        else
		        	if (tmp.contains("WEP"))
		        		tScan.wifis[i] = tScan.wifis[i] + ",WEP" + "," + String.valueOf(lev);
		        	else
		        		tScan.wifis[i] = tScan.wifis[i] + ",NONE" + "," + String.valueOf(lev);
		         
		    if (checkSSID(ssid)) tScan.wifis[i] = tScan.wifis[i] + ",Saved";
	    }
		if (_ba.subExists(tScan.eventname + "_scandone"))
			_ba.raiseEventFromDifferentThread(tScan, null, 0, tScan.eventname + "_scandone", 
					false, new Object[] {tScan.wifis, tScan.wifis.length});
	}	      
}  //Class WifiScanReceiver

/*-----------------------------------------------------------------*/
	      
	private boolean checkSSID(String SSID){
		Context context = BA.applicationContext;
		int i = 0;
		   
		WifiManager wm = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);              
		List<WifiConfiguration> tmp = wm.getConfiguredNetworks();
		   
		if (tmp !=null) {
			for (i = 0; i < tmp.size(); i++)
				if (tmp.get(i).SSID.equals("\""+SSID+"\"")) {
					//BA.Log("SSID= " + tmp.get(i).SSID);
					return true;
				}
		}	   
		return false;
	}
	   
	/**
	 * Returns the BSSID for the selected entry. 
	 * 
	 * EntryNumber - If you display the string array wifis or the array returned in the getWifiList method UNSORTED, you can pass the Position variable from your ListView_Click event.
	 * Otherwise, you must pass the entry number from the string array since that is what is used as a reference.
	 */
	public String getBSSID(int EntryNumber) {	   
		return Results.get(EntryNumber).BSSID;
	}
	   
	/**
	 * Disconnects the current wifi AP connection
	 * 
	 * Returns True on success
	 */
	public boolean Disconnect() {
		Context context = BA.applicationContext;
		WifiManager wifiManager = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);   
		return wifiManager.disconnect();		   
	}
	   
	/**
	 * Returns a String containing info about the entry. A startScan must have already been done. The string contains the SSID, the BSSID, and the capabilities of the network access point.
	 * Each item is separated by a comma (SSID,BSSID,CapabilityString). The capabilities part can be quite long.
	 * 
	 * EntryNumber - If you display the string array wifis or the array returned in the getWifiList method UNSORTED, you can pass the Position variable from your ListView_Click event.
	 * Otherwise, you must pass the entry number from the string array since that is what is used as a reference.
	 * 
	 * Returns a string of comma separated entries of SSID,BSSID, and capability string. Returns an empty string on error. 
	 */
	public String WifiCap(int EntryNumber) {
		String line = "";   
		line = Results.get(EntryNumber).SSID + "," + Results.get(EntryNumber).BSSID + "," + Results.get(EntryNumber).capabilities;   
		return line;
	}
	   
	private WifiConfiguration wfc = new WifiConfiguration();
	private String password = "";
	   
	/**
	 * Checks the stored networks on the device to see if this entry is a previously saved access point
	 * 
	 * EntryNumber - The entry number in the wifis array to check. If you display the string array wifis or the array returned in the getWifiList method UNSORTED, you can pass the Position variable from your ListView_Click event.
	 * Otherwise, you must pass the entry number from the string array since that is what is used as a reference.
	 * 
	 * Returns True if this is a stored network
	 */
	public boolean isStored(int EntryNumber) {
		Context context = BA.applicationContext;
		int i = 0;   
		WifiManager wm = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);              
		List<WifiConfiguration> tmp = wm.getConfiguredNetworks();
		   
		for (i = 0; i < tmp.size(); i++)
			if (tmp.get(i).SSID.equals("\""+Results.get(EntryNumber).SSID+"\"")) {
				BA.Log("Stored SSID= " + tmp.get(i).SSID);
		        return true;
		    }
		return false;
	}
	   
	/**
	 * Connects to a network already configured on the device. The device will use saved values to connect. Updated to handle quoted or unquoted SSID strings.
	 * 
	 * EntryNumber - If you display the string array wifis or the array returned in the getWifiList method UNSORTED, you can pass the Position variable from your ListView_Click event.
	 * Otherwise, you must pass the entry number from the string array since that is what is used as a reference.
	 * 
	 * Returns true if successful
	 */
	public boolean connectToAP(int EntryNumber) {
		int i = 0;		   
		Context context = BA.applicationContext;
		int netId = -1;         
		String ssid = "";
		String bssid = "";
		   
		WifiManager wm = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);              
		   
		if (wm != null && wm.isWifiEnabled()) {
			List<WifiConfiguration> tmp = wm.getConfiguredNetworks();
		   
			if (!Results.get(EntryNumber).SSID.startsWith("\"")) 				   
				ssid = "\""+Results.get(EntryNumber).SSID+"\"";
			else
				ssid = Results.get(EntryNumber).SSID;
		   
			bssid = Results.get(EntryNumber).BSSID;
		   
			BA.Log("Results SSID = " + ssid);
			BA.Log("Results BSSID = " + bssid);
		   
			try {
				for (i = 0; i < tmp.size(); i++) {
					//BA.Log("Stored.networkId = " + tmp.get(i).networkId);
					BA.Log("Stored.ssid = " + tmp.get(i).SSID + " Searched ssid = " + ssid);
					BA.Log("Stored.bssid = " + tmp.get(i).BSSID + " Searched bssid = " + bssid);

					if (tmp.get(i).SSID != null && tmp.get(i).SSID.equals(ssid) || 
						tmp.get(i).BSSID != null && tmp.get(i).BSSID.equals(bssid)) {
						netId = tmp.get(i).networkId;             
						wm.enableNetwork(netId, true);
						BA.Log("Connecting to -----> " + tmp.get(i).SSID + " NetID = " + netId);
						return true;
					}   
				}  //for		   
			}  //try		   
			catch(Exception e) { BA.Log(e.toString()); }
		}
		return false;  
	}
	   
	/**
	 * Builds a list of all saved networks on the device.
	 * 
	 * Returns a comma separated list of SSID's and their BSSID and network ID number.
	 */
	public List<String> listSavedNetworks() {
		Context context = BA.applicationContext;
		List<String> netlst = new ArrayList<String>();
		   
		WifiManager wifiManager = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);              
		   
		for (WifiConfiguration tmp : wifiManager.getConfiguredNetworks()) {
			netlst.add(tmp.SSID + "," + tmp.BSSID + "," + String.valueOf(tmp.networkId));
		}	   
     	return netlst;
	}
	   
	/**
	 * Adds an entry to the WIFI manager and connects to it. This method saves the configuration of the new entry onto the device. 
	 * Use the forgetAP method to delete them. Updated to handle quoted or unquoted SSID strings.
	 * 
	 * EntryNumber: If you display the string array wifis or the array returned in the getWifiList method UNSORTED, 
	 * 				you can pass the Position variable from your ListView_Click event.
	 * 				Otherwise, you must pass the entry number from the string array since that is what is used as a reference.
	 * securityType The Access Point security type (returned in the scan result string array)
	 * Password - The password/phrase for the wifi access point. Pass "" for an open network. Handles WEP HEX or text phrases
	 * 
	 * It is up to you to supply all information. The security type is found in the scan results array.
	 *  
	 * Returns True if the connection is successful
	 */
	public boolean addWifiAPScanned(int EntryNumber, String securityType, String Password) {
    	String ssid = Results.get(EntryNumber).SSID;		   
		//String bssid = Results.get(EntryNumber).BSSID;
		int security;
		password = Password;
		   
		if (securityType.equals("NONE"))
			security = 0;
		else if (securityType.equals("WEP"))
			security = 1;
		else if (securityType.contains("WPA"))
			security = 2;
		else
			return false;
		   
		return addWifiAP(ssid,security,password);
	}
	   
	/**
	 * Delete a stored access point on the device.
	 * 
	 * NetId - The network ID number to delete (returned in the listSavedNetworks method)
	 * 
	 * Returns True if successful
	 */
	public boolean deleteWifiAP(int NetId) {
		Context context = BA.applicationContext;
		boolean ok = false;
		   
		WifiManager wm = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);
		   
		if (wm != null && wm.isWifiEnabled()) {
			ok = wm.removeNetwork(NetId);
		    wm.saveConfiguration();
		}	   
		return ok;
	}
	   
}  //Class MLscan

}  //Class MLwifi

